@@ -19,8 +19,11 @@ public class FileHandle extends SyncHandle implements IIOSeekable, IIOLockable {
 
     FileChannel fc;
     FileLock lock;
+    private boolean truncate = false;
+    private boolean create   = false;
+    private boolean append   = false;
 
-    public static OpenOption[] resolveOpenMode(String mode) {
+    public OpenOption[] resolveOpenMode(String mode) {
         if(mode.length() == 0)
             return null;
 
@@ -51,6 +54,34 @@ public class FileHandle extends SyncHandle implements IIOSeekable, IIOLockable {
             default : return null;
         }
 
+        /* work around differences between Perl 6 and FileChannel.open */
+        List<OpenOption> optsToRemove = new ArrayList<OpenOption>();
+        if (opts.contains(StandardOpenOption.READ)) {
+            if (!opts.contains(StandardOpenOption.WRITE)) {
+                /* TRUNCATE_EXISTING is ignored when the file is opened only for reading. */
+                if (opts.contains(StandardOpenOption.TRUNCATE_EXISTING)) {
+                    truncate = true;
+                    optsToRemove.add(StandardOpenOption.TRUNCATE_EXISTING);
+                }
+                /* CREATE is ignored when the file is opened only for reading. */
+                if (opts.contains(StandardOpenOption.CREATE)) {
+                    create = true;
+                    optsToRemove.add(StandardOpenOption.CREATE);
+                }
+            }
+            /* APPEND may not be used in conjunction with READ. */
+            if (opts.contains(StandardOpenOption.APPEND)) {
+                append = true;
+                optsToRemove.add(StandardOpenOption.APPEND);
+            }
+        }
+        /* APPEND may not be used in conjunction with TRUNCATE_EXISTING. */
+        else if (opts.contains(StandardOpenOption.TRUNCATE_EXISTING) && opts.contains(StandardOpenOption.APPEND)) {
+            append = true;
+            optsToRemove.add(StandardOpenOption.APPEND);
+        }
+        opts.removeAll(optsToRemove);
+
         return opts.toArray(new OpenOption[opts.size()]);
     }
 
@@ -62,6 +93,12 @@ public class FileHandle extends SyncHandle implements IIOSeekable, IIOLockable {
             OpenOption[] opts = resolveOpenMode(mode);
             if(opts == null)
                 ExceptionHandling.dieInternal(tc, "Unhandled file open mode '" + mode + "'");
+            if (truncate)
+                if (Files.exists(p))
+                    Files.write(p, new byte[0]);
+            if (create)
+                if (!Files.exists(p))
+                    Files.createFile(p);
             fc = FileChannel.open(p, opts);
             chan = fc;
             setEncoding(tc, Charset.forName("UTF-8"));
@@ -70,6 +107,34 @@ public class FileHandle extends SyncHandle implements IIOSeekable, IIOLockable {
         }
     }
 
+    public long write(ThreadContext tc, byte[] array) {
+        if (append) {
+            try {
+                fc.position(fc.size());
+            } catch (IOException e) {
+                throw ExceptionHandling.dieInternal(tc, e);
+            }
+            /* Reset readBuffer and eof after calling fc.position. */
+            readBuffer = null;
+            eof = false;
+        }
+        return super.write(tc, array);
+    }
+
+    public long print(ThreadContext tc, String s) {
+        if (append) {
+            try {
+                fc.position(fc.size());
+            } catch (IOException e) {
+                throw ExceptionHandling.dieInternal(tc, e);
+            }
+            /* Reset readBuffer and eof after calling fc.position. */
+            readBuffer = null;
+            eof = false;
+        }
+        return super.print(tc, s);
+    }
+
     public void seek(ThreadContext tc, long offset, long whence) {
         try {
             switch ((int)whence) {
@@ -92,6 +157,8 @@ public class FileHandle extends SyncHandle implements IIOSeekable, IIOLockable {
         }
         /* Reset readBuffer since content is out of sync after fc.position. */
         readBuffer = null;
+        /* Reset eof since it might have changed; needs to be checked anew. */
+        eof = false;
     }
 
     public long tell(ThreadContext tc) {
