@@ -8,9 +8,10 @@ import java.nio.channels.SocketChannel;
 import org.perl6.nqp.runtime.ExceptionHandling;
 import org.perl6.nqp.runtime.ThreadContext;
 
-public class ServerSocketHandle implements IIOBindable {
-    
+public class ServerSocketHandle implements IIOBindable, IIOClosable {
+
     ServerSocketChannel listenChan;
+    public int listenPort;
 
     public ServerSocketHandle(ThreadContext tc) {
         try {
@@ -19,16 +20,17 @@ public class ServerSocketHandle implements IIOBindable {
             ExceptionHandling.dieInternal(tc, e);
         }
     }
-    
+
     public void bind(ThreadContext tc, String host, int port, int backlog) {
         try {
             InetSocketAddress addr = new InetSocketAddress(host, port);
             listenChan.bind(addr, backlog);
+            listenPort = listenChan.socket().getLocalPort();
         } catch (IOException e) {
             throw ExceptionHandling.dieInternal(tc, e);
         }
     }
-    
+
     public SocketHandle accept(ThreadContext tc) {
         try {
             SocketChannel chan = listenChan.accept();
@@ -37,4 +39,12 @@ public class ServerSocketHandle implements IIOBindable {
             throw ExceptionHandling.dieInternal(tc, e);
         }
     }
+
+    public void close(ThreadContext tc) {
+        try {
+            listenChan.close();
+        } catch (IOException e) {
+            throw ExceptionHandling.dieInternal(tc, e);
+        }
+    }
 }
